// vi: set ts=2 shiftwidth=2 expandtab:
//
// VNPForth - Compiled native Forth for IA32 Linux systems
// Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <cstddef>
#include <string>
#include <sstream>
#include <iomanip>

#include "util.h"
#include "dattable.h"
#include "data.h"

// Static data id sequence and instance counter.
int Data::count = 0;
int Data::instances = 0;

// Data constructor, auto-adds to the given data table, and destructor to
// decrement instances and potentially reset count.
Data::Data (size_t size,
            void const * pod,
            int line,
            const std::string & type,
            DataTable & datatable):
  id_ (count++), chunk_ (size), line_ (line), type_ (type), synonym_of_ (-1)
{
  const unsigned char * p = reinterpret_cast<const unsigned char *>(pod);
  chunk_.assign (p, p + size);
  instances++;
  datatable.add (this);
}

Data::~Data ()
{
  if (--instances == 0)
    count = 0;
}

// Pretty-print functions to create data table listing.
const std::string
Data::create_list_header ()
{
  std::ostringstream outs;

  outs << std::setw (8) << "Line" << std::setw (5) << "Id"
       << std::setw (5) << "Aka" << std::setw (8) << "Type"
       << std::setw (5) << "Size" << std::setw (5) << "Data" << std::endl
       << std::setw (8) << "-------" << std::setw (5) << "----"
       << std::setw (5) << "----" << std::setw (8) << "-------"
       << std::setw (5) << "----" << std::setw (5) << "----" << std::endl;

  return outs.str ();
}

const std::string
TextData::create_list_data () const
{
  const std::vector<unsigned char> & chunk = get_chunk ();

  return std::string (reinterpret_cast<const char *>(&chunk[0]), chunk.size ());
}

const std::string
BinaryData::create_list_data () const
{
  const std::vector<unsigned char> & chunk = get_chunk ();

  std::ostringstream outs;
  outs.fill ('0');
  outs << std::hex;
  for (size_t i = 0; i < chunk.size (); ++i)
    outs << std::setw (2) << static_cast<int>(chunk[i]) << " ";

  return outs.str ();
}

const std::string
Data::create_list_entry () const
{
  std::ostringstream outs;

  outs << std::setw (8) << line_
       << std::setw (5) << id_;
  if (synonym_of_ == -1)
    outs << std::setw (5) << " ";
  else
    outs << std::setw (5) << synonym_of_;
  outs << ' ' << leftjustify << std::setw (7) << type_
       << rightjustify << std::setw (5) << chunk_.size () << ' '
       << std::setw (0) << create_list_data () << std::endl;

  return outs.str ();
}
