%{
// vi: set ts=2 shiftwidth=2 expandtab:
//
// VNPForth - Compiled native Forth for IA32 Linux systems
// Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <iostream>

#include "parser.h"

#define YYSTYPE char *
#include "forth.tab.hh"

#define RETURN_WITH_LVALUE(X) \
        do { yylval = yytext; return (X); } while (0)

// External definition of active parser for non-OO yacc/lex code.
extern Parser * _parser;

%}

DIGIT  [0-9]
ODIGIT [0-7]
HDIGIT [0-9A-F]
FCHAR  [!-~]
SPACE  [ \t]

%x code1 code2 code3

%%

<*>"\n"                           _parser->next_line (); REJECT;

[-+]?{DIGIT}+                     RETURN_WITH_LVALUE(INTEGER_LITERAL);
0x{HDIGIT}+                       RETURN_WITH_LVALUE(INTEGER_LITERAL);
0{ODIGIT}+                        RETURN_WITH_LVALUE(INTEGER_LITERAL);

[-+]?{DIGIT}+("."{DIGIT}*)?[Ee][-+]?{DIGIT}* {
                                  RETURN_WITH_LVALUE(FLOAT_LITERAL); }

[-+]?{DIGIT}+"."{DIGIT}*          RETURN_WITH_LVALUE(FLOAT_LITERAL);

'{SPACE}*{FCHAR}+                 RETURN_WITH_LVALUE(OBJECT_ADDRESS);
"["'"]"{SPACE}*{FCHAR}+           RETURN_WITH_LVALUE(OBJECT_ADDRESS);

S\"" "[^\"]*\"                    RETURN_WITH_LVALUE(STRING_LITERAL);
C\"" "[^\"]*\"                    RETURN_WITH_LVALUE(CSTRING);
"."\"" "[^\"]*\"                  RETURN_WITH_LVALUE(PSTRING);

CHAR{SPACE}+{FCHAR}+              RETURN_WITH_LVALUE(CHAR_LITERAL);
\[CHAR\]{SPACE}+{FCHAR}+          RETURN_WITH_LVALUE(CHAR_LITERAL);

:                                 RETURN_WITH_LVALUE(':');
;                                 RETURN_WITH_LVALUE(';');
EXIT                              RETURN_WITH_LVALUE(EXIT);

VARIABLE                          RETURN_WITH_LVALUE(VARIABLE);
CREATE                            RETURN_WITH_LVALUE(CREATE);
ALLOT                             RETURN_WITH_LVALUE(ALLOT);
CONSTANT                          RETURN_WITH_LVALUE(CONSTANT);

IF                                RETURN_WITH_LVALUE(IF);
ELSE                              RETURN_WITH_LVALUE(ELSE);
THEN                              RETURN_WITH_LVALUE(THEN);

DO                                RETURN_WITH_LVALUE(DO);
"?"DO                             RETURN_WITH_LVALUE(QUERY_DO);
LOOP                              RETURN_WITH_LVALUE(LOOP);
"+"LOOP                           RETURN_WITH_LVALUE(PLUS_LOOP);
LEAVE                             RETURN_WITH_LVALUE(LEAVE);

BEGIN                             RETURN_WITH_LVALUE(_BEGIN);
UNTIL                             RETURN_WITH_LVALUE(UNTIL);
AGAIN                             RETURN_WITH_LVALUE(AGAIN);
WHILE                             RETURN_WITH_LVALUE(WHILE);
REPEAT                            RETURN_WITH_LVALUE(REPEAT);

CASE                              RETURN_WITH_LVALUE(CASE);
OF                                RETURN_WITH_LVALUE(OF);
ENDOF                             RETURN_WITH_LVALUE(ENDOF);
ENDCASE                           RETURN_WITH_LVALUE(ENDCASE);

RECURSE                           RETURN_WITH_LVALUE(RECURSE);

CODE                              BEGIN(code1); RETURN_WITH_LVALUE(CODE);
<code1>{
  {SPACE}+
  {FCHAR}+                        BEGIN(code2); RETURN_WITH_LVALUE(WORD);
}
<code2>{
  {SPACE}+
  "\\"{SPACE}[^\n]*\n             BEGIN(code3); _parser->next_line ();
  "\\"\n                          BEGIN(code3); _parser->next_line ();
  "("{SPACE}[^)\n]*")"
  "\n"                            BEGIN(code3);
  .                               { BEGIN(INITIAL);
                                    _parser->parse_error ()
                                      << "expected newline here" << std::endl; }
}
<code3>{
  "\n"
  END-CODE                        BEGIN(INITIAL); RETURN_WITH_LVALUE(END_CODE);
  [^\n]+                          RETURN_WITH_LVALUE(ASM);
}
<code1,code2,code3>.              { BEGIN(INITIAL);
                                    _parser->parse_error ()
                                      << "unknown character '" << yytext <<  "'"
                                      << std::endl; }
<code1,code2,code3><<EOF>>        { BEGIN(INITIAL);
                                    _parser->parse_error ()
                                      << "unexpected EOF in CODE...ENDCODE"
                                      << std::endl;
                                    yyterminate (); }

{SPACE}+
"\\"{SPACE}[^\n]*\n               _parser->next_line ();
"\\"\n                            _parser->next_line ();
"("{SPACE}[^)\n]*")"
"\n"

{FCHAR}+                          RETURN_WITH_LVALUE(WORD);

.                                 { _parser->parse_error ()
                                      << "unknown character '" << yytext << "'"
                                      << std::endl; }

%%
