// vi: set ts=2 shiftwidth=2 expandtab:
//
// VNPForth - Compiled native Forth for IA32 Linux systems
// Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <string>

#include "mangler.h"

// Static class member and locals for entity and hex lookups.
Mangler::EntityTable Mangler::entities_table;
namespace {
  const char * const HEX_CHARACTERS = "0123456789ABCDEF";
  const unsigned int HIGH_WATER_MARK = 4096 - 32;
}

// Static member function to initialize the entities table.
void
Mangler::initialize ()
{
  if (entities_table.empty ())
    {
      entities_table['`']  = "Backtick";
      entities_table['~']  = "Tilde";
      entities_table['!']  = "Store";
      entities_table['@']  = "Fetch";
      entities_table['#']  = "Hash";
      entities_table['$']  = "Dollar";
      entities_table['%']  = "Percent";
      entities_table['^']  = "Caret";
      entities_table['&']  = "Amp";
      entities_table['*']  = "Star";
      entities_table['(']  = "Paren";
      entities_table[')']  = "Closeparen";
      entities_table['-']  = "Minus";
      entities_table['=']  = "Equals";
      entities_table['+']  = "Plus";
      entities_table['[']  = "Leftbracket";
      entities_table[']']  = "Rightbracket";
      entities_table['{']  = "Leftbrace";
      entities_table['}']  = "Rightbrace";
      entities_table['|']  = "Or";
      entities_table[';']  = "Semicolon";
      entities_table[':']  = "Colon";
      entities_table['"']  = "Quote";
      entities_table['\''] = "Tick";
      entities_table[',']  = "Comma";
      entities_table['.']  = "Dot";
      entities_table['/']  = "Slash";
      entities_table['<']  = "Less";
      entities_table['>']  = "Greater";
      entities_table['?']  = "Question";
      entities_table['\\'] = "Backslash";
    }
}

// Static name mangler function.
const std::string
Mangler::mangle (const std::string & s)
{
  initialize ();

  std::string mangled_name = "";
  for (unsigned int i = 0; i < s.length ()
                  && mangled_name.length () < HIGH_WATER_MARK; ++i)
    {
      char c = s[i];

      if (c == '_' || (c >= '0' && c <= '9')
                   || (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'))
        {
          if (i == 0 && c >= '0' && c <= '9')
            mangled_name += 'z';
          mangled_name += c;
          continue;
        }

      if (i > 0)
        mangled_name += '_';

      EntityTableIterator iter = entities_table.find (c);
      if (iter != entities_table.end ())
        {
          mangled_name += iter->second;
          continue;
        }

      mangled_name += std::string ("x")
                      + HEX_CHARACTERS[(c & 0xF0) >> 4]
                      + HEX_CHARACTERS[(c & 0x0F)];
    }

  return mangled_name;
}
