// vi: set ts=2 shiftwidth=2 expandtab:
//
// VNPForth - Compiled native Forth for IA32 Linux systems
// Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef _VNPFORTH_OPCODETABLE_H
#define _VNPFORTH_OPCODETABLE_H

#include <string>
#include <vector>
#include <iostream>

class Options;
class SymbolTable;
class Opcode;

// Opcode table, aggregates opcode pointers into a vector.  The vector
// takes ownership of data objects, and deletes them in its destructor.
class OpcodeTable
{
public:
  OpcodeTable ()
  {
  }
  ~OpcodeTable ();

  void add (Opcode * opcode);
  void create_listing (std::ostream & outs) const;
  void clear ();
  bool is_empty () const;

  void optimize_code ();
  void unreachable_check (std::string source_path) const;

  void generate (std::ostream & outs,
                 const Options & options,
                 const SymbolTable & symtable) const;

private:
  OpcodeTable (const OpcodeTable & table);
  OpcodeTable & operator= (const OpcodeTable & table);

  typedef std::vector<Opcode *> OpcodeTableVector;
  typedef OpcodeTableVector::const_iterator OpcodeTableVectorIterator;
  typedef OpcodeTableVector::iterator OpcodeTableVectorIterator_mutable;

  bool optimize_pair (OpcodeTableVectorIterator_mutable push_iter,
                      OpcodeTableVectorIterator_mutable pop_iter);
  bool optimize_pass ();

  OpcodeTableVector vector_;
};

// Inline function definitions.
inline bool
OpcodeTable::is_empty () const
{
  return vector_.empty ();
}

#endif
