// vi: set ts=2 shiftwidth=2 expandtab:
//
// VNPForth - Compiled native Forth for IA32 Linux systems
// Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef _VNPFORTH_OPTIONS_H
#define _VNPFORTH_OPTIONS_H

class CommandLine;

// Generalized control options and flags.
class Options
{
  friend class CommandLine;

public:
  Options ();

  bool include_debugging () const;
  bool include_profiling () const;
  bool position_independent () const;
  bool optimize_code () const;
  bool save_intermediate () const;
  bool save_assembly () const;
  bool mangle_only () const;

private:
  bool debugging_flag_;
  bool profiling_flag_;
  bool PIC_flag_;
  bool optimize_flag_;
  bool intermediate_flag_;
  bool assembly_flag_;
  bool mangle_flag_;
};

// Inline function definitions.
inline
Options::Options ():
  debugging_flag_ (false), profiling_flag_ (false), PIC_flag_ (false),
  optimize_flag_ (false), intermediate_flag_ (false),
  assembly_flag_ (false), mangle_flag_ (false)
{
}

inline bool
Options::include_debugging () const
{
  return debugging_flag_;
}

inline bool
Options::include_profiling () const
{
  return profiling_flag_;
}

inline bool
Options::position_independent () const
{
  return PIC_flag_;
}

inline bool
Options::optimize_code () const
{
  return optimize_flag_;
}

inline bool
Options::save_intermediate () const
{
  return intermediate_flag_;
}

inline bool
Options::save_assembly () const
{
  return assembly_flag_;
}

inline bool
Options::mangle_only () const
{
  return mangle_flag_;
}

#endif
