// vi: set ts=2 shiftwidth=2 expandtab:
//
// VNPForth - Compiled native Forth for IA32 Linux systems
// Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <cstdio>
#include <cstring>
#include <cerrno>
#include <string>

#include "srcfile.h"

// Destructor, closes any currently open stream.
SourceFile::~SourceFile ()
{
  close ();
}

// File opening and closing.
void
SourceFile::open () throw (std::string)
{
  if (stream_ == 0)
    {
      stream_ = fopen (path_.c_str (), "r");
      if (stream_ == 0)
        throw std::string (strerror (errno));
    }
}

bool
SourceFile::is_open () const
{
  return stream_ != 0;
}

void
SourceFile::close ()
{
  if (stream_ != 0)
    {
      fclose (stream_);
      stream_ = 0;
    }
}


// Copy and assignment constructors.
SourceFile::SourceFile (const SourceFile & source) throw (std::string):
  path_ (source.path_)
{
  if (source.is_open ())
    {
      try {
        open ();
      }
      catch (std::string) {
        throw;
      }
      if (fseek (stream_, ftell (source.stream_), SEEK_SET) == -1)
        {
          close ();
          throw std::string (strerror (errno));
        }
    }
}

SourceFile &
SourceFile::operator= (const SourceFile & source) throw (std::string)
{
  if (&source != this)
    {
      close ();
      path_ = source.path_;
      if (source.is_open ())
        {
          try {
            open ();
          }
          catch (std::string) {
            throw;
          }
          if (fseek (stream_, ftell (source.stream_), SEEK_SET) == -1)
          {
            close ();
            throw std::string (strerror (errno));
          }
        }
    }

  return *this;
}
