// vi: set ts=2 shiftwidth=2 expandtab:
//
// VNPForth - Compiled native Forth for IA32 Linux systems
// Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef _VNPFORTH_STACK_H
#define _VNPFORTH_STACK_H

#include <string>

// Stack class, defines three stacks, and their push and pop functions.
class Stack
{
public:
  const std::string get_name () const;
  const std::string get_push_function () const;
  const std::string get_pop_function () const;
  bool equals (const Stack & stack) const;

  static const Stack DataStack;
  static const Stack ReturnStack;
  static const Stack FloatStack;

private:
  Stack (const char * name,
         const char * push_function,
         const char * pop_function);

  const std::string name_;
  const std::string push_function_;
  const std::string pop_function_;
};

// Inline function definitions.
inline const std::string
Stack::get_name () const
{
  return name_;
}

inline const std::string
Stack::get_push_function () const
{
  return push_function_;
}

inline const std::string
Stack::get_pop_function () const
{
  return pop_function_;
}

inline bool
Stack::equals (const Stack & stack) const
{
  return name_ == stack.name_;
}

inline
Stack::Stack (const char * name,
              const char * push_function,
              const char * pop_function):
  name_ (name), push_function_ (push_function), pop_function_ (pop_function)
{
}

#endif
