// vi: set ts=2 shiftwidth=2 expandtab:
//
// VNPForth - Compiled native Forth for IA32 Linux systems
// Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <string>
#include <sstream>
#include <iomanip>

#include "util.h"
#include "mangler.h"
#include "symtable.h"
#include "symbol.h"

// Static symbol id sequence and instance counter.
int Symbol::count = 0;
int Symbol::instances = 0;

// Ostream inserter for variable symbol enumerations. 
namespace {
  std::ostream &
  operator<< (std::ostream & outs,
              VariableSymbol::Units units)
  {
    switch (units)
      {
      case VariableSymbol::CELL:
        outs << "Cell";
        break;
      case VariableSymbol::CHARACTER:
        outs << "Char";
        break;
      }
    return outs;
  }
}

// Symbol constructor, mangles and auto-adds to the given symbol table, and
// destructor, decrements instances and resets count if necessary.
Symbol::Symbol (const std::string & name,
                int line,
                const std::string type,
                SymbolTable & symtable):
  name_ (Mangler::mangle (name)), id_ (count++), line_ (line), type_ (type)
{
  instances++;
  symtable.add (this);
}

Symbol::~Symbol ()
{
  if (--instances == 0)
    count = 0;
}

// Pretty-print functions to create symbol table listing.
const std::string
Symbol::create_list_header ()
{
  std::ostringstream outs;

  outs << std::setw (8) << "Line" << std::setw (5) << "Id"
       << std::setw (9) << "Type" << std::setw (5) << "Size"
       << std::setw (6) << "Units" << " "
       << std::setw (0) << "Name" << std::endl
       << std::setw (8) << "-------" << std::setw (5) << "----"
       << std::setw (9) << "--------" << std::setw (5) << "----"
       << std::setw (6) << "-----" << " "
       << std::setw (0) << "----" << std::endl;

  return outs.str ();
}

const std::string
Symbol::create_list_entry () const
{
  std::ostringstream outs;

  outs << std::setw (8) << get_line ()
       << std::setw (5) << get_id () << ' '
       << leftjustify << std::setw (8) << get_type ()
       << rightjustify << std::setw (5) << " "
       << std::setw (6) << " "
       << ' ' << std::setw (0) << get_name () << std::endl;

  return outs.str ();
}

const std::string
VariableSymbol::create_list_entry () const
{
  std::ostringstream outs;

  outs << std::setw (8) << get_line ()
       << std::setw (5) << get_id () << ' '
       << leftjustify << std::setw (8) << get_type ()
       << rightjustify << std::setw (5) << size_
       << ' ' << std::setw (5) << units_
       << ' ' << std::setw (0) << get_name () << std::endl;

  return outs.str ();
}
