#!/bin/awk -f
# vi: set ts=2 shiftwidth=2 expandtab:
#
# VNPForth - Compiled native Forth for IA32 Linux systems
# Copyright (C) 2005  Simon Baldwin (simon_baldwin@yahoo.com)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Simple AWK script to produce documentation suitable for processing into
# man pages from a Forth source file.
#

# Initialize the states, arrays, and indices.
BEGIN {
  VARIABLE_DECLARATION    = "VARIABLE"
  CREATE_DECLARATION      = "CREATE"
  WORD_DECLARATION        = ":"
  CODEWORD_DECLARATION    = "CODE"

  VARIABLE_TAG            = "V"
  VARIABLE_SIZETAG        = "B"
  WORD_TAG                = "W"
  STACKDOCUMENTATION_TAG  = "S"
  DOCUMENTATION_TAG       = "D"
  END_TAG                 = "E"
}

# Ignore all blank lines in the file
/^[[:space:]]*$/ {
  next
}

# Handle every other line in the file individually.
{
  # Look for VARIABLE/CREATE declarations, and word or CODEWORDs.
  if ($1 == VARIABLE_DECLARATION || $1 == CREATE_DECLARATION \
      || $1 == WORD_DECLARATION || $1 == CODEWORD_DECLARATION)
    {
      # If the variable name or word begins with "__", we suppress this
      # declaration.  This hides purely private and internal symbols that are
      # not meant for public consumption, for example, __f_abort.
      if (match ($2, /^__/) > 0)
        next;

      # Output the variable's name or word defined.  This is the token after
      # "VARIABLE", "CREATE", ":", or "CODEWORD".
      if ($1 == VARIABLE_DECLARATION || $1 == CREATE_DECLARATION)
        printf ("%s %s\n", VARIABLE_TAG, $2)
      else
        printf ("%s %s\n", WORD_TAG, $2)

      # If a variable, output size information.
      if ($1 == CREATE_DECLARATION)
        {
          if ($4 == "CELLS" || $4 == "CHARS")
            printf ("%s %s %s\n", VARIABLE_SIZETAG, $3, $4)
          else
            printf ("%s %s CHARS\n", VARIABLE_SIZETAG, $3)
        }
      else
        if ($1 == VARIABLE_DECLARATION)
          printf ("%s 1 CELL\n", VARIABLE_SIZETAG)

      # Find any description, and output that too.  The description we are
      # interested in is inside (...)'s, and there may be more than one (...)
      # element to handle on the line.
      if (index ($0, "( ") != 0)
        {
          # Remove the first '( ' and the last ' )' parts.
          sub (/^[^\(]*\([[:space:]]/, "", $0)
          sub (/[[:space:]]\)[^\)]*$/, "", $0)

          # Split the remaining string on the ' ) ( ' pattern.
          count = split ($0, doclines, \
                         /[[:space:]]\)[[:space:]]+\([[:space:]]/ );

          # Print out the doclines array, one line per element, up to, but
          # excluding, the last line.  These are the stack effects
          # documentation.
          for (i = 1; i <= count - 1; i++)
            printf ("%s %s\n", STACKDOCUMENTATION_TAG, doclines[i]);

          # Look for evidence of stack effects documentation on the last
          # documentation element.
          line = doclines[count]
          if (match (line, /^[^,]*--[^,]*,[[:space:]]*/) > 0)
            {
              sline = substr (line, RSTART, RLENGTH)
              line  = substr (line, RLENGTH + 1)
              sub (/[[:space:]]*,[[:space:]]$/, "", sline)
              printf ("%s %s\n", STACKDOCUMENTATION_TAG, sline)
            }
          printf ("%s %s\n", DOCUMENTATION_TAG, line)
        }

      # End this documentation section.
      printf ("%s\n", END_TAG);
    }
}
